/*
 * QUOTA    An implementation of the diskquota system for the LINUX
 *          operating system. QUOTA is implemented using the BSD systemcall
 *          interface as the means of communication with the user level.
 *          Should work for all filesystems because of integration into the
 *          VFS layer of the operating system.
 *          This is based on the Melbourne quota system wich uses both user and
 *          group quota files.
 *
 *          This part does the lookup of the info.
 *
 * Version: $Id: rquota_server.c,v 2.9 1996/11/17 16:59:46 mvw Exp mvw $
 *
 * Author:  Marco van Wieringen <mvw@planets.elm.net>
 *
 *          This program is free software; you can redistribute it and/or
 *          modify it under the terms of the GNU General Public License
 *          as published by the Free Software Foundation; either version
 *          2 of the License, or (at your option) any later version.
 */
#include <dirent.h>
#include <paths.h>
#include <stdio.h>
#include <string.h>

#include <linux/quota.h>

#include <sys/file.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <asm/types.h>

#include <rpc/rpc.h>

#include "rquota.h"
#include "common.h"
#include "quotasys.h"
#include "quota_io.h"

#define TYPE_EXTENDED	0x01
#define ACTIVE		0x02

/*
 * Global unix authentication credentials.
 */
extern struct authunix_parms *unix_cred;

static int in_group (gid_t *gids, u_int len, gid_t gid)
{
	int cnt = 0;

	while (cnt < len) {
		if (gids[cnt] == gid)
			return 1;
		cnt++;
	}
	return 0;
}

static void dqblk2rquota(rquota *rq, struct mem_dqblk *dqb)
{
	rq->rq_bhardlimit = dqb->dqb_bhardlimit;
	rq->rq_bsoftlimit = dqb->dqb_bsoftlimit;
	rq->rq_curblocks = toqb(dqb->dqb_curspace);
	rq->rq_fhardlimit = dqb->dqb_ihardlimit;
	rq->rq_fsoftlimit = dqb->dqb_isoftlimit;
	rq->rq_curfiles = dqb->dqb_curinodes;
	rq->rq_btimeleft = dqb->dqb_btime;
	rq->rq_ftimeleft = dqb->dqb_itime;
}

static getquota_rslt *getquotainfo(int flags, caddr_t *argp, struct svc_req *rqstp)
{
	static getquota_rslt result;
	union {
		getquota_args *args;
		ext_getquota_args *ext_args;
	} arguments;
	struct dquot dquot;
	char *pathname;
	char devbuf[MAXDEVLEN], pathbuf[MAXFILENAMELEN];
	int type;
	qid_t id;

	/*
	* First check authentication.
	*/
	if (flags & TYPE_EXTENDED) {
		arguments.ext_args = (ext_getquota_args *)argp;
		id = arguments.ext_args->gqa_id;
		type = arguments.ext_args->gqa_type;
		pathname = arguments.ext_args->gqa_pathp;

		if (type == USRQUOTA && unix_cred->aup_uid && unix_cred->aup_uid != id) {
			result.status = Q_EPERM;
			return &result;
		}

		if (type == GRPQUOTA && unix_cred->aup_uid && unix_cred->aup_gid != id &&
			 !in_group((gid_t *)unix_cred->aup_gids, unix_cred->aup_len, id)) {
			result.status = Q_EPERM;
			return &result;
		}
	} else {
		arguments.args = (getquota_args *)argp;
		id = arguments.args->gqa_uid;
		type = USRQUOTA;
		pathname = arguments.args->gqa_pathp;

		if (unix_cred->aup_uid && unix_cred->aup_uid != id) {
			result.status = Q_EPERM;
			return &result;
		}
	}

	init_mount(&pathname, 1);
	if (next_mount(kern2userquota(type), devbuf, pathbuf, NULL)) {
		init_io(pathbuf, devbuf, kern2userquota(type));
		if (!QIO_ENABLED && (flags & ACTIVE)) {
			result.status = Q_NOQUOTA;	
			return &result;
		}
		read_dquot(id, &dquot);
		free_io();
		free_mount();

		result.status = Q_OK;	
		result.getquota_rslt_u.gqr_rquota.rq_active = QIO_ENABLED ? TRUE : FALSE;
		result.getquota_rslt_u.gqr_rquota.rq_bsize = QUOTABLOCK_SIZE;
		dqblk2rquota(&result.getquota_rslt_u.gqr_rquota, &dquot.dq_dqb);

		return(&result);
	}
	free_mount();
	result.status = Q_NOQUOTA;	
	return(&result);
}

getquota_rslt *rquotaproc_getquota_1(getquota_args *argp, struct svc_req *rqstp)
{
	return getquotainfo(0, (caddr_t *)argp, rqstp);
}

getquota_rslt *rquotaproc_getactivequota_1(getquota_args *argp, struct svc_req *rqstp)
{
	return getquotainfo(ACTIVE, (caddr_t *)argp, rqstp);
}

getquota_rslt *rquotaproc_getquota_2(ext_getquota_args *argp, struct svc_req *rqstp)
{
	return getquotainfo(TYPE_EXTENDED, (caddr_t *)argp, rqstp);
}

getquota_rslt *rquotaproc_getactivequota_2(ext_getquota_args *argp, struct svc_req *rqstp)
{
	return getquotainfo(TYPE_EXTENDED | ACTIVE, (caddr_t *)argp, rqstp);
}
